<?php
/**
 * inputobj_general.php - The general search input which contains selections for myclasses,
 * 							  all classes, grade and department/subject-area
 *
 * @author $Author: dtong $
 * @version $Id: inputobj_general.php,v 1.19 2012/04/07 19:04:27 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/
if ( ! defined('BASEPATH')) exit('No direct script access allowed');
require_once ('inputobj_base.php');

class inputobj_general extends inputobj_base {
//const SELECT_ALL = 'all';
//const SELECT_IGNORE = 'ignore';
//const SELECT_SIZE = 17;
const SELECT_SEPARATOR = '----------';

private static $dropdown_width_short = array('course' => 230, 'department' => 200, 'grade' => 150);
private static $dropdown_width_long = array('course' => 350, 'department' => 250, 'grade' => 150);

private $dropdown_width;
private $select_size = 17;

	function __construct($data) {
		parent::__construct($data);
		$this->CI->load->model('menu/menu_model');
		$this->CI->load->model('unit/unit_model');
		$this->CI->load->helper('form');
		osa_load_lang('mapping');
		if ( ($size = $this->get_templatevalue('_select_size')) !== FALSE ) {
			$this->select_size = $size;
		}
		$this->dropdown_width = &self::$dropdown_width_long;
	}

	function render() {
		$html = '<nobr>' .
				  $this->get_mycourses() . ' &nbsp; ' .
				  $this->get_allcourses() . ' &nbsp; ' .
				  $this->get_allgrades()  . ' &nbsp; ' .
				  $this->get_alldepartments();
		return $html . "</nobr><div style='font-size:9px;margin-top:5px'>" . lang('map_multiselect_instruction') . '</div>';
	}

	function render_courses() {
		return $this->get_mycourses() . ' &nbsp; ' . $this->get_allcourses() . ' &nbsp; ';
	}

	public function get_alldepartments($fieldname='_departments') {
		static $cache = array();

		$fieldname = $fieldname . '[]';
		$gradeid = $this->get_gradebytemplate();
		if ( $gradeid !== FALSE ) {
			$key = implode(',', $gradeid);
			if ( isset($cache[$key]) ) {
				$depts = &$cache[$key];
			}
			else {
				$depts = $this->CI->menu_model->get_departmentbygrade($gradeid);
				$cache[$key] = &$depts;
			}
		}
		else {
			if ( isset($cache['nograde']) ) {
				$depts = &$cache['nograde'];
			}
			else {
				$depts = $this->CI->menu_model->getalldepartment();
				$cache['nograde'] = &$depts;
			}
		}

		if ( empty($depts) ) {
			$depts = array();
		}

		$selections = array('null' => lang('map_nonedept'), '' => self::SELECT_SEPARATOR);

		$default = array();
		foreach ($depts as $dept) {
			$selections[$dept['id']] = &$dept['name'];
			$default[] = &$dept['id'];
		}

		$width = $this->dropdown_width['department'];
		if(osa_value($this->data, '_deptsel_default_all', FALSE) === TRUE) {
			return form_dropdown($fieldname, $selections, $default, 'size="' . $this->select_size . '" multiple="multiple" style="max-width:' . $width . 'px"');
		}
		else {
			return form_dropdown($fieldname, $selections, 'xxxZZZ', 'size="' . $this->select_size . '" multiple="multiple" style="max-width:' . $width . 'px"');
		}

	}

	public function get_allgrades($fieldname='_grades') {
		static $cache = array();
		$fieldname = $fieldname . '[]';
		$gradeid = $this->get_gradebytemplate();
		if ( $gradeid !== FALSE ){
			$key = implode(',', $gradeid);
			if (isset($cache[$key])) {
				$grades = &$cache[$key];
			}
			else {
				$grades = $this->CI->menu_model->getgrade($gradeid);
				$cache[$key] = &$grades;
			}
		}
		else {
			if ( isset($cache['nograde']) ) {
				$grades = &$cache['nograde'];
			}
			else {
				$grades = $this->CI->menu_model->getgrade();
				$cache['nograde'] = &$grades;
			}
		}

		if ( empty($grades) ) {
			$grades = array();
		}

		$selections = array('null' => lang('map_nonegrade'), '' => self::SELECT_SEPARATOR);
		foreach ($grades as $grade) {
			$selections[$grade['id']] = &$grade['name'];
		}
		$width = $this->dropdown_width['grade'];
		return form_dropdown($fieldname, $selections, 'xxxZZZ', 'size="' . $this->select_size . '" multiple="multiple" style="max-width:' . $width . 'px"');
	}

	private function get_mycourses($defaultall=FALSE, $fieldname='_mycourses') {
		static $courses = NULL;
		$fieldname = $fieldname . '[]';
		/*if ( $this->has_mycourses() ) {
			return "<input type=\"checkbox\" name=\"{$fieldname}\" value=\"1\" /> " . lang('map_mycourses');
		}*/
		$user = $this->CI->login_model->getLogin();
		if ( $courses === NULL ) {
			$courses = $this->CI->menu_model->mycourses($user->id);
			//Use natural sort
			osa_array_natsort($courses, 'name');
		}
		if ( !empty($courses) && count($courses) > 0 ) {
			$this->dropdown_width = &self::$dropdown_width_short;
			/*$selections = array(CMS_SELECT_ALL => lang('map_nonemycourses'),
				self::SELECT_IGNORE => lang('map_selectignore'), '' => self::SELECT_SEPARATOR);*/

			if ( count($courses) > 1) {
				$ids = '';
				$comma = '';
				foreach ($courses as $course) {
					$ids .= $comma . $course['id'];
					$comma = ',';
				}
				$selections = array('null' => lang('map_nonemycourses'),'' => self::SELECT_SEPARATOR,
										  "$ids"=>lang('map_allmycourses'));
			}
			else {
				$selections = array('null' => lang('map_nonemycourses'),'' => self::SELECT_SEPARATOR);
			}
			foreach ($courses as $course) {
				$selections[$course['id']] = &$course['name'];
			}
			if ( $defaultall )
				$selected = CMS_SELECT_ALL;
			else
				$selected = 'xxxZZZ';
			$width = $this->dropdown_width['course'];
			return form_dropdown($fieldname, $selections, $selected, 'size="' . $this->select_size . '" multiple="multiple" style="max-width:' . $width . 'px"');
		}
		return FALSE;
	}

	private function get_allcourses($defaultall=FALSE, $fieldname='_courses') {
		static $cache = array();
		$fieldname = $fieldname . '[]';
		$gradeid = $this->get_gradebytemplate();
		$courses = FALSE;
		if ( $gradeid !== FALSE ){
			$key = implode(',', $gradeid);
			if ( isset($cache[$key]) ) {
				$courses = &$cache[$key];
			}
			else {
				$courses = $this->CI->menu_model->get_courses($gradeid);
				//Use our own natural sort to sort the course names
				osa_array_natsort($courses, 'course');
				$cache[$key] = &$courses;
			}
		}
		else {
			if ( isset($cache['nograde']) ) {
				$courses = &$cache['nograde'];
			}
			else {
				$courses = $this->CI->unit_model->get_enable_courses();
				//Use our own natural sort to sort the course names
				osa_array_natsort($courses, 'course');
				$cache['nograde'] = &$courses;
			}
		}

		if ( empty($courses) ) {
			$courses = array();
		}
		$tmparray = array('null' => lang('map_nonecourse'),'' => self::SELECT_SEPARATOR);
		//Do copy and retain indexes, array_merge() resets indexes
		foreach ( $courses as $record ) {
			$tmparray[$record->courseid] = &$record->course;
		}
		if ( $defaultall )
			$selected = CMS_SELECT_ALL;
		else
			$selected = 'xxxZZZ';
		$width = $this->dropdown_width['course'];
		return form_dropdown($fieldname, $tmparray, $selected, 'size="' . $this->select_size . '" multiple="multiple" style="max-width:' . $width . 'px"');
	}

	//Gets all the grades according to a template (ubd, pyp, myp etc.)
	private function get_gradebytemplate() {
		// static $db_grades = NULL;
		if ( isset($this->data['_template']) && $this->data['_template'] != FALSE ) {
			// $unit_template can be a string like 'ubd'.
			$unit_template = $this->CI->config->item('cms_unit_templates');
			if ( !is_array($unit_template) ) {
				// The caller will use all the grades.
				return FALSE;
			}
			$template = $this->data['_template'];
			$gradeid = array();
			if (is_array($unit_template) && is_array($template)){
				foreach ($unit_template as $key=>$value){
					foreach ($template as $index){
						if (in_array($index,$value))
							$gradeid[] = $key;
					}
				}
			}
			/*
			 * The whole purpose this function is to find a small set of grades according to the defined template
			 * but this code always return the full grade list. Added code at the top of the function to return false when
			 * cms_unit_templates is not an array. Issue 133 does not apply to this function.
			//Start of (issue: 133)
			if ( $db_grades === NULL ) {
				$db_grades = $this->CI->menu_model->getgrade();
			}
			if ( $db_grades ) {
				//Find grades that are not defined in cms_unit_templates and add them
				foreach ($db_grades as $record) {
					if ( !array_key_exists($record['id'], $unit_template) ) {
						$gradeid[] = $record['id'];
					}
				}
			}
			//End of (issue: 133)
			 */
			if ( empty($gradeid) ) {
				return FALSE;
			}
			return $gradeid;
		}
		return FALSE;
	}

/*
	private function has_mycourses() {
		$user = $this->CI->login_model->getLogin();
		$courses = $this->CI->menu_model->mycourses($user->id);
		if ( count($courses) == 0 )
			return FALSE;
		return TRUE;
	}
*/
}
