<?php
/**
 * Paginig_handle_main.php
 *
 * The main class to manage all the paging handles.
 * This is mainly used to disable template page paging numbers based on conditions.
 *
 * @author $Author: dtong $
 * @version $Id: Paging_handle_main.php,v 1.3 2011/09/29 09:28:53 dtong Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/

Class Paging_handle_main {
	private $CI;

	function __construct() {
		$this->CI = & get_instance();
	}
	/*
	 * $template - The template object obtained from factory
	 * $data_input - The original $data passed to Unitmain constructor
	 * $pagenumber - The page number being checked
	 */
	function run($pagenumber, &$template, &$data_input) {
		if ( empty($template) || !is_object($template) || !osa_is_int($pagenumber) ) {
			osa_errorlog(__METHOD__ . " - Invalid input parameter current_page '{$pagenumber} or non-object template'");
			return osa_buildererror();
		}
		$ret = FALSE;
		$has_handle = FALSE;
		if ( isset($template->page_check) ) {
			//Only runs if there is a page check array
			if ( !is_array($template->page_check) ) {
				osa_errorlog(__METHOD__ . " - page_check is not an array.", $template->page_check);
				return osa_buildererror();
			}
			if ( isset($template->page_check[$pagenumber]) && is_array($template->page_check[$pagenumber]) ) {
				if ( array_key_exists('_handle', $template->page_check[$pagenumber]) ) {
					//Single handle and no need to be wrap in a array in the template file
					//So, put it in an array
					$handles = array($template->page_check[$pagenumber]);
				}
				else {
					//Multiple handles in an array
					$handles = & $template->page_check[$pagenumber];
				}
				$check_all = TRUE;
				if ( array_key_exists('_check_all', $handles) ) {
					$check_all = $handles['_check_all'];
				}
				foreach ($handles as $key=>$handle_check_array) {
					//Check to see if the page number requires checking
					//$handle_check_array = & $template->page_check[$pagenumber];
					if ( $key === '_check_all' ) {
						continue;
					}
					$has_handle = TRUE;
					if ( !isset($handle_check_array['_handle']) ) {
						osa_errorlog(__METHOD__ . " - Missing _handle index for page $pagenumber.", $handle_check_array);
						return osa_buildererror();
					}
					$handlename = 'paging_' . $handle_check_array['_handle'];
					//Load the handl object
					if (!@include_once("{$handlename}.php")) {
						osa_errorlog(__METHOD__ . " - Missing file {$handlename}.php.");
						return osa_buildererror();
					}
					$handleobj = & $this->CI->cms_cache->object_get($handlename);
					if ( empty($handleobj) ) {
						osa_errorlog(__METHOD__ . " - Handle class {$handlename} does not exist");
						return osa_buildererror();
					}
					//We init/reset the object. This allows us to reuse the object
					if ( !$handleobj->reset($pagenumber, $template, $handle_check_array, $data_input) ) {
						return osa_buildererror();
					}
					$handle_ret = $handleobj->invoke();
					if ( !$handle_ret && $check_all ) {
						return FALSE;
					}
					if ( $handle_ret ) {
						$ret = $handle_ret;
					}
				}//foreach
			}
		}
		if ( !$has_handle ) {
			//Never executed a handle so return true
		return TRUE;
	}
		return $ret;
	}
}