<?php
/**
 * access_factory.php - Gets the access properties. Hard coded for now, but can change to DB later.
 *
 * @author $Author: dtong $
 * @version $Id: access_factory.php,v 1.30 2012/05/23 14:39:30 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

class OSA_AccessFactory {
	private static $CI = NULL;

	// Gets the access object. Do not change this object outside of this class.
   public static function factory(&$type, &$guestname) {
   	static $cache = array();
   	if ( isset($cache[$type] ) ) {
   		return $cache[$type];
   	}

   	$obj = new cms_access_config();
      switch ( $type ) {
         case 'admin':
         	//Just give all access to admin
         	foreach($obj as $key=>$value) {
         		$obj->$key = TRUE;
         	}
            break;
         case 'teacher':
            $obj->course_description_read = TRUE;
            $obj->course_description_write = TRUE;
            $obj->course_learningtarget_read = TRUE;
            $obj->course_learningtarget_write = TRUE;
            $obj->unit_read = TRUE;
            $obj->unit_write = TRUE;
            $obj->version_access = TRUE;
            $obj->homepage_access = TRUE;
            $obj->goal_read = TRUE;
            $obj->survey_write = TRUE;
            $obj->non_guest_access = TRUE;
				$obj->mapping = TRUE;
				$obj->ict_sample_upload = TRUE;
				$obj->file_download = TRUE;
				$obj->allow_unit_download = TRUE;
				$obj->allow_course_download = TRUE;
				$obj->allow_site_download = TRUE;
				$obj->site_resource_read = TRUE;
            break;
         case $guestname:
         	$obj->course_learningtarget_read = TRUE;
            $obj->course_description_read = TRUE;
            $obj->unit_read = TRUE;
            $obj->homepage_access = TRUE;
            // $obj->file_download = TRUE;
            self::mod_guest($obj);
            break;
         default:
         	$obj = FALSE;
      }
      $cache[$type] = $obj;
      return $obj;
   }

   // Put all the access modfications for the guest user in this function.
   private static function mod_guest(&$obj) {
   	if ( self::$CI === NULL ) {
   		self::$CI = get_instance();
   	}
   	/*
   	$file_download = FALSE;
   	// Check config if allowing guest to download
   	$download_all = !self::$CI->config->item('cms_guest_disable_download_all');
   	if ( $download_all || !self::$CI->config->item('cms_guest_disable_unit_download') ) {
   		$obj->allow_unit_download = TRUE;
   		$file_download = TRUE;
   	}
   	if ( $download_all || !self::$CI->config->item('cms_guest_disable_course_download') ) {
   		$obj->allow_course_download = TRUE;
   		$file_download = TRUE;
   	}
   	if ( $download_all || !self::$CI->config->item('cms_guest_disable_site_download') ) {
   		$obj->allow_site_download = TRUE;
   		$file_download = TRUE;
   	}
   	$obj->file_download = $file_download;
		*/
   	if ( !self::$CI->config->item('cms_guest_disable_download_all') ) {
   		if ( !self::$CI->config->item('cms_guest_disable_unit_download') ) {
   			$obj->allow_unit_download = TRUE;
   		}
   		if ( !self::$CI->config->item('cms_guest_disable_course_download') ) {
   			$obj->allow_course_download = TRUE;
   		}
   		if ( !self::$CI->config->item('cms_guest_disable_site_download') ) {
   			$obj->allow_site_download = TRUE;
   		}
   		if ( $obj->allow_unit_download || $obj->allow_course_download || $obj->allow_site_download ) {
   			$obj->file_download = TRUE;
   		}
   	}
   	//Check for site resource access
   	if ( self::$CI->config->item('cms_guest_enable_site_resource_read') ) {
   		$obj->site_resource_read = TRUE;
   	}
   }
}

class cms_access_config {
	//Admin function can assign teachers to be a course editor
	public $admin_assign_editor = FALSE;
	//Admin function can manage user accounts
	public $admin_system_management = FALSE;
	//admin function can write to all units, targets and descriptions
	public $admin_unit_write = FALSE;
	//Can read all units including editing units, not implemented yet
	public $admin_unit_read = FALSE;
	public $admin_learningtarget_write = FALSE;
	public $admin_course_description_write = FALSE;
	//Can read course description
	public $course_description_read = FALSE;
	//User can write course description to courses that are assigned as editor
	public $course_description_write = FALSE; //This is course based and needs editor assignment
	//Users can read all learning targets
	public $course_learningtarget_read = FALSE;
	//Users can write to learning targets for courses assigned as editor
	public $course_learningtarget_write = FALSE; //This is course based and needs editor assignment
	//Can read all units
	public $unit_read = FALSE;
	//User can write to units to courses assigned as editors
	public $unit_write = FALSE;
	//User can access archived unit versions
	public $version_access = FALSE;
	public $homepage_access = FALSE;
	public $ict_sample_upload = FALSE;
	//Admin can access login as another user
	public $admin_loginas = FALSE;
	public $course_hide_access = FALSE;
	public $unit_hide_access = FALSE;
	public $admin_goal_write = FALSE;
	public $admin_goal_readonly = FALSE;
	//Can edit/write any ict, gc or tail
	public $admin_ict_write = FALSE;
	//Users can read goal
	public $goal_read = FALSE;
	public $goal_admin_management = FALSE;
	public $survey_write = FALSE;
	public $survey_admin_write = FALSE;
	public $non_guest_access = FALSE;
	public $mapping = FALSE;
	public $file_download = FALSE;
	// Users can download uploaded resources in unit page
	public $allow_unit_download = FALSE;
	// Users can download uploaded resources in course description page
	public $allow_course_download = FALSE;
	// Users can download uploaded resources in site resource page
	public $allow_site_download = FALSE;
	// Users can access to write site resources
	public $site_resource_write = FALSE;
	// Users can access to read site resources
	public $site_resource_read = FALSE;
}